package gliparameterizer;

import java.awt.*;
import javax.swing.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.File;

/**
 * <p>Title: GLI Light Ray Tracer</p>
 * <p>Description: Determines which trees intercept given light rays.</p>
 * <p>Copyright: Copyright (c) 2004 Charles D. Canham</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */

public class GUI extends JFrame implements ActionListener {

  /**Edit box where tree input file name goes*/
  protected JTextField m_jTreeInFile = new JTextField();

  /**Edit box where gli ray input file name goes*/
  protected JTextField m_jGLIInFile = new JTextField();

  /**Edit box where allometry input file name goes*/
  protected JTextField m_jAllometryInFile = new JTextField();

  /**Edit box where output file name goes*/
  protected JTextField m_jOutFile = new JTextField();

  /**Save the directory between browsings*/
  protected File m_oDirectory = null;

  public GUI() {
    super("GLI Ray Analyzer");
    try {
      jbInit();
      pack();
    }
    catch(Exception ex) {
      ex.printStackTrace();
    }
  }

  private void jbInit() throws Exception {
    JPanel jMainPanel = new JPanel();
    jMainPanel.setLayout(new BoxLayout(jMainPanel, BoxLayout.PAGE_AXIS));

    //Filename and browse button for tree file
    JLabel jTempLabel = new JLabel("Tree File Name:");
    jTempLabel.setFont(new ModelFont());
    jTempLabel.setHorizontalAlignment(SwingConstants.LEFT);
    jTempLabel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    JPanel jTempPanel = new JPanel(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
    jTempPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jTempPanel.setBorder(BorderFactory.createEmptyBorder(10, 0, 0, 0));
    jTempPanel.add(jTempLabel);
    jMainPanel.add(jTempPanel);

    m_jTreeInFile.setFont(new ModelFont());
    m_jTreeInFile.setText("");
    m_jTreeInFile.setPreferredSize(new Dimension(300,
        (int) m_jTreeInFile.getPreferredSize().getHeight()));
    JButton jBrowseButton = new JButton("Browse");
    jBrowseButton.setFont(new ModelFont());
    jBrowseButton.addActionListener(this);
    jBrowseButton.setActionCommand("BrowseTree");
    jBrowseButton.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jTempPanel = new JPanel();
    jTempPanel.add(m_jTreeInFile);
    jTempPanel.add(jBrowseButton);
    jTempPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jMainPanel.add(jTempPanel);

    //Filename and browse button for GLI ray file
    jTempLabel = new JLabel("GLI Ray File Name:");
    jTempLabel.setFont(new ModelFont());
    jTempLabel.setHorizontalAlignment(SwingConstants.LEFT);
    jTempLabel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jTempPanel = new JPanel(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
    jTempPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jTempPanel.setBorder(BorderFactory.createEmptyBorder(10, 0, 0, 0));
    jTempPanel.add(jTempLabel);
    jMainPanel.add(jTempPanel);

    m_jGLIInFile.setFont(new ModelFont());
    m_jGLIInFile.setText("");
    m_jGLIInFile.setPreferredSize(new Dimension(300,
        (int) m_jGLIInFile.getPreferredSize().getHeight()));
    jBrowseButton = new JButton("Browse");
    jBrowseButton.setFont(new ModelFont());
    jBrowseButton.addActionListener(this);
    jBrowseButton.setActionCommand("BrowseGLI");
    jBrowseButton.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jTempPanel = new JPanel();
    jTempPanel.add(m_jGLIInFile);
    jTempPanel.add(jBrowseButton);
    jTempPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jMainPanel.add(jTempPanel);

    //Filename and browse button for allometry file
    jTempLabel = new JLabel("Allometry File Name:");
    jTempLabel.setFont(new ModelFont());
    jTempLabel.setHorizontalAlignment(SwingConstants.LEFT);
    jTempLabel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jTempPanel = new JPanel(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
    jTempPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jTempPanel.setBorder(BorderFactory.createEmptyBorder(10, 0, 0, 0));
    jTempPanel.add(jTempLabel);
    jMainPanel.add(jTempPanel);

    m_jAllometryInFile.setFont(new ModelFont());
    m_jAllometryInFile.setText("");
    m_jAllometryInFile.setPreferredSize(new Dimension(300,
        (int) m_jAllometryInFile.getPreferredSize().getHeight()));
    jBrowseButton = new JButton("Browse");
    jBrowseButton.setFont(new ModelFont());
    jBrowseButton.addActionListener(this);
    jBrowseButton.setActionCommand("BrowseAllometry");
    jBrowseButton.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jTempPanel = new JPanel();
    jTempPanel.add(m_jAllometryInFile);
    jTempPanel.add(jBrowseButton);
    jTempPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jMainPanel.add(jTempPanel);

    //Filename and browse button for output file
    jTempLabel = new JLabel("Output File Name:");
    jTempLabel.setFont(new ModelFont());
    jTempLabel.setHorizontalAlignment(SwingConstants.LEFT);
    jTempLabel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jTempPanel = new JPanel(new java.awt.FlowLayout(java.awt.FlowLayout.LEFT));
    jTempPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jTempPanel.setBorder(BorderFactory.createEmptyBorder(10, 0, 0, 0));
    jTempPanel.add(jTempLabel);
    jMainPanel.add(jTempPanel);

    m_jOutFile.setFont(new ModelFont());
    m_jOutFile.setText("");
    m_jOutFile.setPreferredSize(new Dimension(300,
        (int) m_jOutFile.getPreferredSize().getHeight()));
    jBrowseButton = new JButton("Browse");
    jBrowseButton.setFont(new ModelFont());
    jBrowseButton.addActionListener(this);
    jBrowseButton.setActionCommand("BrowseOut");
    jBrowseButton.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jTempPanel = new JPanel();
    jTempPanel.add(m_jOutFile);
    jTempPanel.add(jBrowseButton);
    jTempPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);
    jMainPanel.add(jTempPanel);

    //Run button and Exit button
    JButton jButton = new JButton("Run Analysis");
    jButton.setFont(new ModelFont());
    jButton.addActionListener(this);
    jButton.setActionCommand("Run");
    jTempPanel = new JPanel(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));
    jTempPanel.add(jButton);
    jTempPanel.setAlignmentX(java.awt.Component.LEFT_ALIGNMENT);

    //Close window
    jButton = new JButton("Exit");
    jButton.setFont(new ModelFont());
    jButton.addActionListener(this);
    jButton.setActionCommand("Cancel");
    jTempPanel.add(jButton);

    //Help
    jButton = new JButton("Help");
    jButton.setFont(new ModelFont());
    jButton.addActionListener(this);
    jButton.setActionCommand("Help");
    jTempPanel.add(jButton);

    jMainPanel.add(jTempPanel);

    setContentPane(jMainPanel);

  }

  /**
   * Responds to buttons on the GUI.
   * @param oEvent ActionEvent
   */
  public void actionPerformed(ActionEvent oEvent) {
    if (oEvent.getActionCommand().equals("Run")) {
      try {

        //Make sure we have all four filenames that we need
        String sAllometryInFile = m_jAllometryInFile.getText(),
            sGLIInFile = m_jGLIInFile.getText(),
            sTreeInFile = m_jTreeInFile.getText(),
            sOutFile = m_jOutFile.getText();

        if (sAllometryInFile == null || sAllometryInFile.length() == 0) {
          JOptionPane.showMessageDialog(this,
              "I need a filename for the allometry file.");
          return;
        }

        if (sGLIInFile == null || sGLIInFile.length() == 0) {
          JOptionPane.showMessageDialog(this,
              "I need a filename for the GLI ray file.");
          return;
        }

        if (sTreeInFile == null || sTreeInFile.length() == 0) {
          JOptionPane.showMessageDialog(this,
                                        "I need a filename for the tree file.");
          return;
        }

        if (sOutFile == null || sOutFile.length() == 0) {
          JOptionPane.showMessageDialog(this,
              "I need a filename for the output file.");
          return;
        }

        this.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
        Analyzer oAnalyzer = null;
        oAnalyzer = new Analyzer();
        oAnalyzer.RunAnalysis(sAllometryInFile, sGLIInFile, sTreeInFile,
                              sOutFile);
        this.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
        JOptionPane.showMessageDialog(this, "Analysis Complete.");
        oAnalyzer = null;
      } catch (Exception e) {
        JOptionPane.showMessageDialog(this, e.getMessage(), "ERROR", JOptionPane.ERROR_MESSAGE);
        this.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
      }
    }
    else if (oEvent.getActionCommand().equals("Cancel")) {
      //Close window
      this.setVisible(false);
      this.dispose();
    }
    else if (oEvent.getActionCommand().equals("Help")) {
      Help oHelp = new Help(this);
      oHelp.pack();
      oHelp.setVisible(true);
      oHelp.setLocationRelativeTo(null);
    }
    else if (oEvent.getActionCommand().equals("BrowseTree")) {
      JFileChooser chooser = new JFileChooser();

      //Default to the last directory used, if any
      if (m_oDirectory != null && m_oDirectory.getParentFile() != null && m_oDirectory.getParentFile().exists()) {
        chooser.setCurrentDirectory(m_oDirectory.getParentFile());
      }

      //Set up our file if needed
      int returnVal = chooser.showOpenDialog(this);
      if (returnVal == JFileChooser.APPROVE_OPTION) {
        //User chose a file - what kind?
        File oFile = chooser.getSelectedFile();
        String sFileName = oFile.getAbsolutePath();
        m_jTreeInFile.setText(sFileName);
        m_oDirectory = oFile;
      }
    }
    else if (oEvent.getActionCommand().equals("BrowseGLI")) {
      JFileChooser chooser = new JFileChooser();

      //Default to the last directory used, if any
      if (m_oDirectory != null && m_oDirectory.getParentFile() != null && m_oDirectory.getParentFile().exists()) {
        chooser.setCurrentDirectory(m_oDirectory.getParentFile());
      }

      //Set up our file if needed
      int returnVal = chooser.showOpenDialog(this);
      if (returnVal == JFileChooser.APPROVE_OPTION) {
        //User chose a file - what kind?
        File oFile = chooser.getSelectedFile();
        String sFileName = oFile.getAbsolutePath();
        m_jGLIInFile.setText(sFileName);
        m_oDirectory = oFile;
      }
    }
    else if (oEvent.getActionCommand().equals("BrowseAllometry")) {
      JFileChooser chooser = new JFileChooser();

      //Default to the last directory used, if any
      if (m_oDirectory != null && m_oDirectory.getParentFile() != null && m_oDirectory.getParentFile().exists()) {
        chooser.setCurrentDirectory(m_oDirectory.getParentFile());
      }

      //Set up our file if needed
      int returnVal = chooser.showOpenDialog(this);
      if (returnVal == JFileChooser.APPROVE_OPTION) {
        //User chose a file - what kind?
        File oFile = chooser.getSelectedFile();
        String sFileName = oFile.getAbsolutePath();
        m_jAllometryInFile.setText(sFileName);
        m_oDirectory = oFile;
      }
    }
    else if (oEvent.getActionCommand().equals("BrowseOut")) {
      JFileChooser chooser = new JFileChooser();

      //Default to the last directory used, if any
      if (m_oDirectory != null && m_oDirectory.getParentFile() != null && m_oDirectory.getParentFile().exists()) {
        chooser.setCurrentDirectory(m_oDirectory.getParentFile());
      }

      int returnVal = chooser.showSaveDialog(this);
      if (returnVal == JFileChooser.APPROVE_OPTION) {
        //User chose a file - does it already exist?
        File oFile = chooser.getSelectedFile();

        //Does this file already exist?
        if (oFile.exists()) {
          JOptionPane pane = new JOptionPane();
          returnVal = pane.showConfirmDialog(this,
                                             "This file already exists.  Overwrite?",
                                             "Model",
                                             JOptionPane.YES_NO_OPTION);
          if (returnVal == JOptionPane.YES_OPTION) {
            String sFileName = oFile.getAbsolutePath();
            m_jOutFile.setText(sFileName);
            m_oDirectory = oFile;
          }
        }
        else {
          //File didn't exist
          String sFileName = oFile.getAbsolutePath();
          m_jOutFile.setText(sFileName);
        }
      }
    }
  }
}
