package gliparameterizer;

import javax.swing.UIManager;

/**
 * Main class.
 * <p>Title: GLI Light Ray Tracer</p>
 * <p>Description: Determines which trees intercept given light rays.  This
 * is the main class.</p>
 * <p>
 * This application takes a series of light rays and a set of trees, and
 * determines which trees intercept each light ray.  A ray is defined by an
 * X and Y location, height above the ground, azimuth, and altitude.  A tree
 * is defined by X and Y location, species, and DBH.  There are other pieces
 * of information which identify a tree but are meaningless for the
 * calculations performed in this code.
 * </p>
 * <p>Copyright: Copyright (c) 2004 Charles D. Canham</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */
public class Main {

  public Main() {
  }

  /**
   * Main function.
   * @param args String[] Arguments not needed.
   */
  public static void main(String[] args) {
    try {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());

      GUI oFrame = new GUI();
      oFrame.pack();

      //Center the window
      java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().
          getScreenSize();
      java.awt.Dimension frameSize = oFrame.getSize();
      if (frameSize.height > screenSize.height) {
        frameSize.height = screenSize.height;
      }
      if (frameSize.width > screenSize.width) {
        frameSize.width = screenSize.width;
      }
      oFrame.setLocation( (screenSize.width - frameSize.width) / 2,
                         (screenSize.height - frameSize.height) / 2);
      oFrame.setVisible(true);
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

}
