package gliparameterizer;

import junit.framework.*;

/**
 * <p>Title: GLI Light Ray Tracer</p>
 * <p>Description: Determines which trees intercept given light rays.</p>
 * <p>Copyright: Copyright (c) 2004 Charles D. Canham</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */

public class TestAllTrees
    extends TestCase {

  protected void setUp() throws Exception {
    super.setUp();
  }

  protected void tearDown() throws Exception {
    super.tearDown();
  }

  public void testGetCanopyRadiusExponent() throws Exception {
    //Create a tree population object
    AllTrees oAllTrees = new AllTrees(200, 200);

    //Set it up with allometry data
    oAllTrees.mp_oAllometry.add(new AllometryPackage("ALCLAT", (float) 0.073,
        (float) 1.0, (float) 27.3, (float) 0.025, (float) 0.477));
    oAllTrees.mp_oAllometry.add(new AllometryPackage("BUCCAP", (float) 0.06,
        (float) 0.9, (float) 24.57, (float) 0.048, (float) 0.55));
    oAllTrees.mp_oAllometry.add(new AllometryPackage("CASARB", (float) 0.09,
        (float) 0.8, (float) 30, (float) 0.029, (float) 0.54));
    oAllTrees.mp_oAllometry.add(new AllometryPackage("DACEXC", (float) 0.075,
        (float) 1.1, (float) 23.81, (float) 0.061, (float) 0.576));
    oAllTrees.m_oOther = (new AllometryPackage("OTHER", (float) 0.074,
                                               (float) 1.5, (float) 23.84,
                                               (float) 0.049, (float) 0.497));

    //Case:  get named value
    assertEquals(1.0, oAllTrees.GetCanopyRadiusExponent("ALCLAT"), 0.001);
    assertEquals(0.9, oAllTrees.GetCanopyRadiusExponent("BUCCAP"), 0.001);
    assertEquals(0.8, oAllTrees.GetCanopyRadiusExponent("CASARB"), 0.001);
    assertEquals(1.1, oAllTrees.GetCanopyRadiusExponent("DACEXC"), 0.001);

    //Case:  unknown species when "Other" category is present
    assertEquals(1.5, oAllTrees.GetCanopyRadiusExponent("PICKLE"), 0.001);

    //Create a tree population object - no "other" this time
    oAllTrees = null;
    oAllTrees = new AllTrees(200, 200);

    //Set it up with allometry data
    oAllTrees.mp_oAllometry.add(new AllometryPackage("ALCLAT", (float) 0.073,
        (float) 1.0, (float) 27.3, (float) 0.025, (float) 0.477));
    oAllTrees.mp_oAllometry.add(new AllometryPackage("BUCCAP", (float) 0.06,
        (float) 0.9, (float) 24.57, (float) 0.048, (float) 0.55));
    oAllTrees.mp_oAllometry.add(new AllometryPackage("CASARB", (float) 0.09,
        (float) 0.8, (float) 30, (float) 0.029, (float) 0.54));
    oAllTrees.mp_oAllometry.add(new AllometryPackage("DACEXC", (float) 0.075,
        (float) 1.1, (float) 23.81, (float) 0.061, (float) 0.576));

    //Case:  get named value
    assertEquals(1.0, oAllTrees.GetCanopyRadiusExponent("ALCLAT"), 0.001);
    assertEquals(0.9, oAllTrees.GetCanopyRadiusExponent("BUCCAP"), 0.001);
    assertEquals(0.8, oAllTrees.GetCanopyRadiusExponent("CASARB"), 0.001);
    assertEquals(1.1, oAllTrees.GetCanopyRadiusExponent("DACEXC"), 0.001);

    //Case:  unknown species when "Other" category is present - should fail
    try {
      oAllTrees.GetCanopyRadiusExponent("PICKLE");
      fail("GetCanopyRadiusExponent() didn't catch unrecognized species.");
    } catch (Exception oErr) {
      ;
    }
  }

  public void testGetMaxAllTreeHeight() throws Exception {
    //Create a tree population object
    AllTrees oAllTrees = new AllTrees(200, 200);

    //Set it up with allometry data - max is not in "Other"
    oAllTrees.mp_oAllometry.add(new AllometryPackage("ALCLAT", (float) 0.073,
        (float) 1.0, (float) 27.3, (float) 0.025, (float) 0.477));
    oAllTrees.mp_oAllometry.add(new AllometryPackage("BUCCAP", (float) 0.06,
        (float) 0.9, (float) 24.57, (float) 0.048, (float) 0.55));
    oAllTrees.mp_oAllometry.add(new AllometryPackage("CASARB", (float) 0.09,
        (float) 0.8, (float) 30, (float) 0.029, (float) 0.54));
    oAllTrees.mp_oAllometry.add(new AllometryPackage("DACEXC", (float) 0.075,
        (float) 1.1, (float) 23.81, (float) 0.061, (float) 0.576));
    oAllTrees.m_oOther = (new AllometryPackage("OTHER", (float) 0.074,
                                               (float) 1.5, (float) 23.84,
                                               (float) 0.049, (float) 0.497));

    double fExpectedValue = 30,
        fActualValue = oAllTrees.GetMaxAllTreeHeight();
    assertEquals(fExpectedValue, fActualValue, 0.0001);

    //Max is in "Other"
    oAllTrees = null;
    oAllTrees = new AllTrees(200, 200);
    //Set it up with allometry data - max is not in "Other"
    oAllTrees.mp_oAllometry.add(new AllometryPackage("ALCLAT", (float) 0.073,
        (float) 1.0, (float) 27.3, (float) 0.025, (float) 0.477));
    oAllTrees.mp_oAllometry.add(new AllometryPackage("BUCCAP", (float) 0.06,
        (float) 0.9, (float) 24.57, (float) 0.048, (float) 0.55));
    oAllTrees.mp_oAllometry.add(new AllometryPackage("CASARB", (float) 0.09,
        (float) 0.8, (float) 30, (float) 0.029, (float) 0.54));
    oAllTrees.mp_oAllometry.add(new AllometryPackage("DACEXC", (float) 0.075,
        (float) 1.1, (float) 23.81, (float) 0.061, (float) 0.576));
    oAllTrees.m_oOther = (new AllometryPackage("OTHER", (float) 0.074,
                                               (float) 1.5, (float) 43.84,
                                               (float) 0.049, (float) 0.497));

    fExpectedValue = 43.84;
    fActualValue = oAllTrees.GetMaxAllTreeHeight();
    assertEquals(fExpectedValue, fActualValue, 0.0001);

  }

  public void testAddTree() throws Exception {
    //Create a tree population object
    AllTrees oAllTrees = new AllTrees(200, 200);
    //Set it up with allometry data
    oAllTrees.mp_oAllometry.add(new AllometryPackage("ALCLAT", (float) 0.073,
        (float) 1.0, (float) 27.3, (float) 0.025, (float) 0.477));
    oAllTrees.mp_oAllometry.add(new AllometryPackage("BUCCAP", (float) 0.06,
        (float) 0.9, (float) 24.57, (float) 0.048, (float) 0.55));
    oAllTrees.mp_oAllometry.add(new AllometryPackage("CASARB", (float) 0.09,
        (float) 0.8, (float) 30, (float) 0.029, (float) 0.54));
    oAllTrees.mp_oAllometry.add(new AllometryPackage("DACEXC", (float) 0.075,
        (float) 1.1, (float) 23.81, (float) 0.061, (float) 0.576));
    oAllTrees.m_oOther = (new AllometryPackage("OTHER", (float) 0.074,
                                               (float) 1.5, (float) 23.84,
                                               (float) 0.049, (float) 0.497));

    //Make sure it adds trees into the right place
    double fX = 3.0,
        fY = 0.1;
    Tree oTree = new Tree("Test", fX, fY, 10.0, 101, (byte) 2, oAllTrees);
    oAllTrees.AddTree(oTree);
    assertTrue(oAllTrees.mp_oAllTrees[0][0].size() == 1);

    fX = 45.2;
    fY = 98.2;
    oTree = new Tree("Test", fX, fY, 10.0, 101, (byte) 2, oAllTrees);
    oAllTrees.AddTree(oTree);
    assertTrue(oAllTrees.mp_oAllTrees[0][0].size() == 1);
    assertTrue(oAllTrees.mp_oAllTrees[5][12].size() == 1);

    fX = 198.2;
    fY = 195.2;
    oTree = new Tree("Test", fX, fY, 10.0, 101, (byte) 2, oAllTrees);
    oAllTrees.AddTree(oTree);
    assertTrue(oAllTrees.mp_oAllTrees[0][0].size() == 1);
    assertTrue(oAllTrees.mp_oAllTrees[5][12].size() == 1);
    assertTrue(oAllTrees.mp_oAllTrees[24][24].size() == 1);

    fX = 42.2;
    fY = 100.5;
    oTree = new Tree("Test", fX, fY, 10.0, 101, (byte) 2, oAllTrees);
    oAllTrees.AddTree(oTree);
    assertTrue(oAllTrees.mp_oAllTrees[0][0].size() == 1);
    assertTrue(oAllTrees.mp_oAllTrees[5][12].size() == 2);
    assertTrue(oAllTrees.mp_oAllTrees[24][24].size() == 1);

    //Error processing
    try {
      fX = -45.2;
      fY = 98.2;
      oTree = new Tree("Test", fX, fY, 10.0, 101, (byte) 2, oAllTrees);
      oAllTrees.AddTree(oTree);
      fail("AllTrees.AddTree didn't catch negative X value.");
    }
    catch (Exception e) {
      ;
    }

    try {
      fX = 245.2;
      fY = 98.2;
      oTree = new Tree("Test", fX, fY, 10.0, 101, (byte) 2, oAllTrees);
      oAllTrees.AddTree(oTree);
      fail("AllTrees.AddTree didn't catch X value too big for plot.");
    }
    catch (Exception e) {
      ;
    }

    try {
      fY = -45.2;
      fX = 98.2;
      oTree = new Tree("Test", fX, fY, 10.0, 101, (byte) 2, oAllTrees);
      oAllTrees.AddTree(oTree);
      fail("AllTrees.AddTree didn't catch negative Y value.");
    }
    catch (Exception e) {
      ;
    }

    try {
      fY = 245.2;
      fX = 98.2;
      oTree = new Tree("Test", fX, fY, 10.0, 101, (byte) 2, oAllTrees);
      oAllTrees.AddTree(oTree);
      fail("AllTrees.AddTree didn't catch Y value too big for plot.");
    }
    catch (Exception e) {
      ;
    }
  }

}
