package gliparameterizer;

import junit.framework.*;
import java.io.FileWriter;
import java.io.File;

/**
 * Tests the Analyzer class.
 * <p>Copyright: Copyright (c) 2004 Charles D. Canham</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */

public class TestAnalyzer
    extends TestCase {

  protected void setUp() throws Exception {
    super.setUp();
  }

  protected void tearDown() throws Exception {
    super.tearDown();
  }

  /**
   * Tests the reading of the allometry files.
   * @throws Exception if a test is failed.
   */
  public void testReadAllometryFile() throws Exception {
    Analyzer oMain = null;
    String sFileName;
    AllometryPackage oAllometry;

    //Case:  normal file, no "other" category
    oMain = new Analyzer();
    sFileName = WriteTestAllometryFile1();
    oMain.ReadAllometryFile(sFileName);
    assertEquals(320, oMain.m_oAllTrees.m_iPlotXLength);
    assertEquals(500, oMain.m_oAllTrees.m_iPlotYLength);
    assertEquals(4, oMain.m_oAllTrees.mp_oAllometry.size());
    assertTrue(null == oMain.m_oAllTrees.m_oOther);
    oAllometry = (AllometryPackage) oMain.m_oAllTrees.mp_oAllometry.elementAt(0);
    assertEquals("ALCLAT", oAllometry.m_sSpecies);
    assertEquals(0.073, oAllometry.m_fRadiusCoefficient, 0.0001);
    assertEquals(1, oAllometry.m_fRadiusExponent, 0.0001);
    assertEquals(27.3, oAllometry.m_fMaxHeight, 0.0001);
    assertEquals(0.025, oAllometry.m_fHeightConst, 0.0001);
    assertEquals(0.477, oAllometry.m_fCanopyDepthCoeff, 0.0001);

    oAllometry = (AllometryPackage) oMain.m_oAllTrees.mp_oAllometry.elementAt(1);
    assertEquals("BUCCAP", oAllometry.m_sSpecies);
    assertEquals(0.06, oAllometry.m_fRadiusCoefficient, 0.0001);
    assertEquals(1, oAllometry.m_fRadiusExponent, 0.0001);
    assertEquals(24.57, oAllometry.m_fMaxHeight, 0.0001);
    assertEquals(0.048, oAllometry.m_fHeightConst, 0.0001);
    assertEquals(0.55, oAllometry.m_fCanopyDepthCoeff, 0.0001);

    oAllometry = (AllometryPackage) oMain.m_oAllTrees.mp_oAllometry.elementAt(2);
    assertEquals("CASARB", oAllometry.m_sSpecies);
    assertEquals(0.09, oAllometry.m_fRadiusCoefficient, 0.0001);
    assertEquals(1, oAllometry.m_fRadiusExponent, 0.0001);
    assertEquals(30, oAllometry.m_fMaxHeight, 0.0001);
    assertEquals(0.029, oAllometry.m_fHeightConst, 0.0001);
    assertEquals(0.54, oAllometry.m_fCanopyDepthCoeff, 0.0001);

    oAllometry = (AllometryPackage) oMain.m_oAllTrees.mp_oAllometry.elementAt(3);
    assertEquals("DACEXC", oAllometry.m_sSpecies);
    assertEquals(0.075, oAllometry.m_fRadiusCoefficient, 0.0001);
    assertEquals(1, oAllometry.m_fRadiusExponent, 0.0001);
    assertEquals(23.81, oAllometry.m_fMaxHeight, 0.0001);
    assertEquals(0.061, oAllometry.m_fHeightConst, 0.0001);
    assertEquals(0.576, oAllometry.m_fCanopyDepthCoeff, 0.0001);

    boolean bDelete = new File(sFileName).delete();
    if (bDelete == false) {
      fail("Bad file stream management - couldn't delete file \"" + sFileName +
           "\".");
    }

    //Case:  normal file, with "other" category
    oMain = null;
    oMain = new Analyzer();
    sFileName = WriteTestAllometryFile2();
    oMain.ReadAllometryFile(sFileName);
    assertEquals(320, oMain.m_oAllTrees.m_iPlotXLength);
    assertEquals(500, oMain.m_oAllTrees.m_iPlotYLength);
    assertEquals(4, oMain.m_oAllTrees.mp_oAllometry.size());
    assertTrue(null != oMain.m_oAllTrees.m_oOther);
    oAllometry = (AllometryPackage) oMain.m_oAllTrees.mp_oAllometry.elementAt(0);
    assertEquals("ALCLAT", oAllometry.m_sSpecies);
    assertEquals(0.073, oAllometry.m_fRadiusCoefficient, 0.0001);
    assertEquals(1, oAllometry.m_fRadiusExponent, 0.0001);
    assertEquals(27.3, oAllometry.m_fMaxHeight, 0.0001);
    assertEquals(0.025, oAllometry.m_fHeightConst, 0.0001);
    assertEquals(0.477, oAllometry.m_fCanopyDepthCoeff, 0.0001);

    oAllometry = (AllometryPackage) oMain.m_oAllTrees.mp_oAllometry.elementAt(1);
    assertEquals("BUCCAP", oAllometry.m_sSpecies);
    assertEquals(0.06, oAllometry.m_fRadiusCoefficient, 0.0001);
    assertEquals(0.9, oAllometry.m_fRadiusExponent, 0.0001);
    assertEquals(24.57, oAllometry.m_fMaxHeight, 0.0001);
    assertEquals(0.048, oAllometry.m_fHeightConst, 0.0001);
    assertEquals(0.55, oAllometry.m_fCanopyDepthCoeff, 0.0001);

    oAllometry = (AllometryPackage) oMain.m_oAllTrees.mp_oAllometry.elementAt(2);
    assertEquals("CASARB", oAllometry.m_sSpecies);
    assertEquals(0.09, oAllometry.m_fRadiusCoefficient, 0.0001);
    assertEquals(0.8, oAllometry.m_fRadiusExponent, 0.0001);
    assertEquals(30, oAllometry.m_fMaxHeight, 0.0001);
    assertEquals(0.029, oAllometry.m_fHeightConst, 0.0001);
    assertEquals(0.54, oAllometry.m_fCanopyDepthCoeff, 0.0001);

    oAllometry = (AllometryPackage) oMain.m_oAllTrees.mp_oAllometry.elementAt(3);
    assertEquals("DACEXC", oAllometry.m_sSpecies);
    assertEquals(0.075, oAllometry.m_fRadiusCoefficient, 0.0001);
    assertEquals(1.1, oAllometry.m_fRadiusExponent, 0.0001);
    assertEquals(23.81, oAllometry.m_fMaxHeight, 0.0001);
    assertEquals(0.061, oAllometry.m_fHeightConst, 0.0001);
    assertEquals(0.576, oAllometry.m_fCanopyDepthCoeff, 0.0001);

    oAllometry = oMain.m_oAllTrees.m_oOther;
    assertEquals("OTHER", oAllometry.m_sSpecies);
    assertEquals(0.074, oAllometry.m_fRadiusCoefficient, 0.0001);
    assertEquals(1.5, oAllometry.m_fRadiusExponent, 0.0001);
    assertEquals(23.84, oAllometry.m_fMaxHeight, 0.0001);
    assertEquals(0.049, oAllometry.m_fHeightConst, 0.0001);
    assertEquals(0.497, oAllometry.m_fCanopyDepthCoeff, 0.0001);

    bDelete = new File(sFileName).delete();
    if (bDelete == false) {
      fail("Bad file stream management - couldn't delete file \"" + sFileName +
           "\".");
    }

    //Error processing
    try {
      oMain.ReadAllometryFile(WriteTestAllometryErrorFile1());
      fail("Main.ReadAllometryFile failed to catch missing X plot length.");

      bDelete = new File(sFileName).delete();
      if (bDelete == false) {
        fail("Bad file stream management - couldn't delete file \"" + sFileName +
             "\".");
      }
    }
    catch (Exception e) {
      ;
    }

    try {
      oMain.ReadAllometryFile(WriteTestAllometryErrorFile2());
      fail("Main.ReadAllometryFile failed to catch missing Y plot length.");

      bDelete = new File(sFileName).delete();
      if (bDelete == false) {
        fail("Bad file stream management - couldn't delete file \"" + sFileName +
             "\".");
      }
    }
    catch (Exception e) {
      ;
    }

    try {
      oMain.ReadAllometryFile(WriteTestAllometryErrorFile3());
      fail("Main.ReadAllometryFile failed to catch truncated file.");

      bDelete = new File(sFileName).delete();
      if (bDelete == false) {
        fail("Bad file stream management - couldn't delete file \"" + sFileName +
             "\".");
      }
    }
    catch (Exception e) {
      ;
    }
  }

  public void testReadGLIRayFile() throws Exception {
    Analyzer oMain = null;
    String sFileName;
    GLIRay oRay = null;

    oMain = new Analyzer();
    sFileName = WriteTestGLIFile1();
    oMain.ReadGLIRayFile(sFileName);
    assertEquals(6, oMain.mp_oGLIRays.size());

    oRay = (GLIRay) oMain.mp_oGLIRays.elementAt(0);
    assertEquals(0.34906585, oRay.m_fAltitude, 0.0001);
    assertEquals(0.087266463, oRay.m_fAzimuth, 0.0001);
    assertEquals("30311", oRay.m_sSubquad);
    assertEquals("0", oRay.m_sLight);
    assertEquals(125, oRay.m_fX, 0.0001);
    assertEquals(325, oRay.m_fY, 0.0001);
    assertEquals(1.25, oRay.m_fHeight, 0.0001);

    oRay = (GLIRay) oMain.mp_oGLIRays.elementAt(1);
    assertEquals(0.087266463, oRay.m_fAltitude, 0.0001);
    assertEquals(3.141592654, oRay.m_fAzimuth, 0.0001);
    assertEquals("30311", oRay.m_sSubquad);
    assertEquals("1.1", oRay.m_sLight);
    assertEquals(225, oRay.m_fX, 0.0001);
    assertEquals(425, oRay.m_fY, 0.0001);
    assertEquals(1.75, oRay.m_fHeight, 0.0001);

    oRay = (GLIRay) oMain.mp_oGLIRays.elementAt(2);
    assertEquals(0.436332313, oRay.m_fAltitude, 0.0001);
    assertEquals(6.021385919, oRay.m_fAzimuth, 0.0001);
    assertEquals("301", oRay.m_sSubquad);
    assertEquals("0", oRay.m_sLight);
    assertEquals(225, oRay.m_fX, 0.0001);
    assertEquals(425, oRay.m_fY, 0.0001);
    assertEquals(2, oRay.m_fHeight, 0.0001);

    oRay = (GLIRay) oMain.mp_oGLIRays.elementAt(3);
    assertEquals(1.047197551, oRay.m_fAltitude, 0.0001);
    assertEquals(0, oRay.m_fAzimuth, 0.0001);
    assertEquals("30311", oRay.m_sSubquad);
    assertEquals("0", oRay.m_sLight);
    assertEquals(160, oRay.m_fX, 0.0001);
    assertEquals(490, oRay.m_fY, 0.0001);
    assertEquals(1.25, oRay.m_fHeight, 0.0001);

    oRay = (GLIRay) oMain.mp_oGLIRays.elementAt(4);
    assertEquals(0.436332313, oRay.m_fAltitude, 0.0001);
    assertEquals(2.408554368, oRay.m_fAzimuth, 0.0001);
    assertEquals("30311", oRay.m_sSubquad);
    assertEquals("0", oRay.m_sLight);
    assertEquals(225, oRay.m_fX, 0.0001);
    assertEquals(425, oRay.m_fY, 0.0001);
    assertEquals(1.25, oRay.m_fHeight, 0.0001);

    oRay = (GLIRay) oMain.mp_oGLIRays.elementAt(5);
    assertEquals(0.523598776, oRay.m_fAltitude, 0.0001);
    assertEquals(0.34906585, oRay.m_fAzimuth, 0.0001);
    assertEquals("30311", oRay.m_sSubquad);
    assertEquals("0", oRay.m_sLight);
    assertEquals(265, oRay.m_fX, 0.0001);
    assertEquals(440, oRay.m_fY, 0.0001);
    assertEquals(1.25, oRay.m_fHeight, 0.0001);

    boolean bDelete = new File(sFileName).delete();
    if (bDelete == false) {
      fail("Bad file stream management - couldn't delete file \"" + sFileName +
           "\".");
    }

    //Error processing
    try {
      oMain.ReadGLIRayFile(WriteTestGLIErrorFile1());
      fail("Main.ReadGLIRayFile failed to catch truncated file.");

      bDelete = new File(sFileName).delete();
      if (bDelete == false) {
        fail("Bad file stream management - couldn't delete file \"" + sFileName +
             "\".");
      }
    }
    catch (Exception e) {
      ;
    }

  }

  public void testReadTreeFile() throws Exception {
    Analyzer oMain = null;
    String sFileName;
    Tree oTree = null;

    oMain = new Analyzer();
    //Do allometry file so there will be setup
    sFileName = WriteTestAllometryFile2();
    oMain.ReadAllometryFile(sFileName);
    boolean bDelete = new File(sFileName).delete();
    if (bDelete == false) {
      fail("Bad file stream management - couldn't delete file \"" + sFileName +
           "\".");
    }
    sFileName = WriteTestTreeFile1();
    oMain.ReadTreeFile(sFileName);
    assertEquals(1, oMain.m_oAllTrees.mp_oAllTrees[6][5].size());
    oTree = (Tree) oMain.m_oAllTrees.mp_oAllTrees[6][5].elementAt(0);
    assertEquals(15067, oTree.m_iTag);
    assertEquals("BUCCAP", oTree.m_sSpecies);
    assertEquals(82.0, oTree.m_fDBH, 0.0001);
    assertEquals( (byte) 0, oTree.m_iDamageCode);
    assertEquals(49.61, oTree.m_fX, 0.0001);
    assertEquals(45.16, oTree.m_fY, 0.0001);

    assertEquals(1, oMain.m_oAllTrees.mp_oAllTrees[6][13].size());
    oTree = (Tree) oMain.m_oAllTrees.mp_oAllTrees[6][13].elementAt(0);
    assertEquals(16873, oTree.m_iTag);
    assertEquals("BUCCAP", oTree.m_sSpecies);
    assertEquals(61.5, oTree.m_fDBH, 0.0001);
    assertEquals( (byte) 2, oTree.m_iDamageCode);
    assertEquals(50.3, oTree.m_fX, 0.0001);
    assertEquals(104.57, oTree.m_fY, 0.0001);

    assertEquals(1, oMain.m_oAllTrees.mp_oAllTrees[10][14].size());
    oTree = (Tree) oMain.m_oAllTrees.mp_oAllTrees[10][14].elementAt(0);
    assertEquals(4622, oTree.m_iTag);
    assertEquals("BUCCAP", oTree.m_sSpecies);
    assertEquals(59.3, oTree.m_fDBH, 0.0001);
    assertEquals( (byte) 0, oTree.m_iDamageCode);
    assertEquals(86.96, oTree.m_fX, 0.0001);
    assertEquals(113.91, oTree.m_fY, 0.0001);

    assertEquals(1, oMain.m_oAllTrees.mp_oAllTrees[10][11].size());
    oTree = (Tree) oMain.m_oAllTrees.mp_oAllTrees[10][11].elementAt(0);
    assertEquals(12756, oTree.m_iTag);
    assertEquals("ALCLAT", oTree.m_sSpecies);
    assertEquals(22.4, oTree.m_fDBH, 0.0001);
    assertEquals( (byte) 0, oTree.m_iDamageCode);
    assertEquals(87.77, oTree.m_fX, 0.0001);
    assertEquals(89.53, oTree.m_fY, 0.0001);

    assertEquals(1, oMain.m_oAllTrees.mp_oAllTrees[11][19].size());
    oTree = (Tree) oMain.m_oAllTrees.mp_oAllTrees[11][19].elementAt(0);
    assertEquals(6626, oTree.m_iTag);
    assertEquals("ANDINE", oTree.m_sSpecies);
    assertEquals(25.2, oTree.m_fDBH, 0.0001);
    assertEquals( (byte) 0, oTree.m_iDamageCode);
    assertEquals(88.98, oTree.m_fX, 0.0001);
    assertEquals(152.93, oTree.m_fY, 0.0001);

    bDelete = new File(sFileName).delete();
    if (bDelete == false) {
      fail("Bad file stream management - couldn't delete file \"" + sFileName +
           "\".");
    }

    //Error processing
    try {
      oMain.ReadTreeFile(WriteTestTreeErrorFile1());
      fail("Main.ReadTreeFile failed to catch truncated file.");

      bDelete = new File(sFileName).delete();
      if (bDelete == false) {
        fail("Bad file stream management - couldn't delete file \"" + sFileName +
             "\".");
      }
    }
    catch (Exception e) {
      ;
    }
  }

  /**
   * Writes a file that has 4 allometry packages, with no "other".  Canopy
   * radius exponents are all equal to 1.
   * @return String Filename.
   * @throws java.io.IOException if the file could not be written.
   */
  protected String WriteTestAllometryFile1() throws java.io.IOException {
    String sFileName = "testfile1.txt";

    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("X plot length:\t320\n");
    oOut.write("Y plot length:\t500\n");
    oOut.write("Species\tRad Coeff\tRad Exp\tMaxHt\tHtconst\tDepth coeff\n");
    oOut.write("ALCLAT\t0.073\t1\t27.3\t0.025\t0.477\n");
    oOut.write("BUCCAP\t0.06\t1\t24.57\t0.048\t0.55\n");
    oOut.write("CASARB\t0.09\t1\t30\t0.029\t0.54\n");
    oOut.write("DACEXC\t0.075\t1\t23.81\t0.061\t0.576\n");

    oOut.close();

    return sFileName;
  }

  /**
   * Writes a file that has 4 allometry packages, with an "other".
   * @return String Filename.
   * @throws java.io.IOException if the file could not be written.
   */
  protected String WriteTestAllometryFile2() throws java.io.IOException {
    String sFileName = "testfile1.txt";

    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("X plot length:\t320\n");
    oOut.write("Y plot length:\t500\n");
    oOut.write("Species\tRad Coeff\tRad Exp\tMaxHt\tHtconst\tDepth coeff\n");
    oOut.write("ALCLAT\t0.073\t1\t27.3\t0.025\t0.477\n");
    oOut.write("BUCCAP\t0.06\t0.9\t24.57\t0.048\t0.55\n");
    oOut.write("CASARB\t0.09\t0.8\t30\t0.029\t0.54\n");
    oOut.write("DACEXC\t0.075\t1.1\t23.81\t0.061\t0.576\n");
    oOut.write("OTHER\t0.074\t1.5\t23.84\t0.049\t0.497\n");

    oOut.close();

    return sFileName;
  }

  /**
   * Writes a file that is missing X plot length.
   * @return String Filename.
   * @throws java.io.IOException if the file could not be written.
   */
  protected String WriteTestAllometryErrorFile1() throws java.io.IOException {
    String sFileName = "testfile1.txt";

    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("X plot length:\n");
    oOut.write("Y plot length:\t500\n");
    oOut.write("Species\tRad Coeff\tRad Exp\tMaxHt\tHtconst\tDepth coeff\n");
    oOut.write("ALCLAT\t0.073\t1\t27.3\t0.025\t0.477\n");
    oOut.write("BUCCAP\t0.06\t0.9\t24.57\t0.048\t0.55\n");
    oOut.write("CASARB\t0.09\t0.8\t30\t0.029\t0.54\n");
    oOut.write("DACEXC\t0.075\t1.1\t23.81\t0.061\t0.576\n");
    oOut.write("OTHER\t0.074\t1.5\t23.84\t0.049\t0.497\n");

    oOut.close();

    return sFileName;
  }

  /**
   * Writes a file that is missing Y plot length.
   * @return String Filename.
   * @throws java.io.IOException if the file could not be written.
   */
  protected String WriteTestAllometryErrorFile2() throws java.io.IOException {
    String sFileName = "testfile1.txt";

    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("X plot length:\t500\n");
    oOut.write("Y plot length:\n");
    oOut.write("Species\tRad Coeff\tRad Exp\tMaxHt\tHtconst\tDepth coeff\n");
    oOut.write("ALCLAT\t0.073\t1\t27.3\t0.025\t0.477\n");
    oOut.write("BUCCAP\t0.06\t0.9\t24.57\t0.048\t0.55\n");
    oOut.write("CASARB\t0.09\t0.8\t30\t0.029\t0.54\n");
    oOut.write("DACEXC\t0.075\t1.1\t23.81\t0.061\t0.576\n");
    oOut.write("OTHER\t0.074\t1.5\t23.84\t0.049\t0.497\n");

    oOut.close();

    return sFileName;
  }

  /**
   * Writes a file that truncates in the middle.
   * @return String Filename.
   * @throws java.io.IOException if the file could not be written.
   */
  protected String WriteTestAllometryErrorFile3() throws java.io.IOException {
    String sFileName = "testfile1.txt";

    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("X plot length:\t500\n");
    oOut.write("Y plot length:\t400\n");
    oOut.write("Species\tRad Coeff\tRad Exp\tMaxHt\tHtconst\tDepth coeff\n");
    oOut.write("ALCLAT\t0.073\t1\t27.3\t0.025\t0.477\n");
    oOut.write("BUCCAP\t0.06\t0.9\t24.57\t");

    oOut.close();

    return sFileName;
  }

  /**
   * Writes a file that has 5 trees.
   * @return String Filename.
   * @throws java.io.IOException if the file could not be written.
   */
  protected String WriteTestTreeFile1() throws java.io.IOException {
    String sFileName = "testfile1.txt";

    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("TAG\tSPP\tDBH\tDAMCODE\tXm\tYm\n");
    oOut.write("15067\tBUCCAP\t82\t0\t49.61\t45.16\n");
    oOut.write("16873\tBUCCAP\t61.5\t2\t50.3\t104.57\n");
    oOut.write("4622\tBUCCAP\t59.3\t0\t86.96\t113.91\n");
    oOut.write("12756\tALCLAT\t22.4\t0\t87.77\t89.53\n");
    oOut.write("6626\tANDINE\t25.2\t0\t88.98\t152.93\n");

    oOut.close();

    return sFileName;
  }

  /**
   * Writes a file that truncates in the middle.
   * @return String Filename.
   * @throws java.io.IOException if the file could not be written.
   */
  protected String WriteTestTreeErrorFile1() throws java.io.IOException {
    String sFileName = "testfile1.txt";

    FileWriter oOut = new FileWriter(sFileName);

    oOut.write("TAG\tSPP\tDBH\tDAMCODE\tXm\tYm\n");
    oOut.write("15067\tBUCCAP\t82\t0\t49.61\t345.16\n");
    oOut.write("16873\tBUCCAP\t61.5\t2\t50.3\t404.57\n");
    oOut.write("4622\tBUCCAP\t59.3");

    oOut.close();

    return sFileName;
  }

  /**
   * Writes a GLI file that has 6 points.
   * @return String Filename.
   * @throws java.io.IOException if the file could not be written.
   */
  protected String WriteTestGLIFile1() throws java.io.IOException {
    String sFileName = "testfile1.txt";

    FileWriter oOut = new FileWriter(sFileName);
    oOut.write("Alt\tAzimuth\tSubquad\tLight\tX\tY\tHeight\n");
    oOut.write("20\t5\t30311\t0\t125\t325\t1.25\n");
    oOut.write("5\t180\t30311\t1.1\t225\t425\t1.75\n");
    oOut.write("25\t345\t301\t0\t225\t425\t2\n");
    oOut.write("60\t0\t30311\t0\t160\t490\t1.25\n");
    oOut.write("25\t138\t30311\t0\t225\t425\t1.25\n");
    oOut.write("30\t20\t30311\t0\t265\t440\t1.25\n");

    oOut.close();

    return sFileName;
  }

  /**
   * Writes a GLI file that truncates in the middle.
   * @return String Filename.
   * @throws java.io.IOException if the file could not be written.
   */
  protected String WriteTestGLIErrorFile1() throws java.io.IOException {
    String sFileName = "testfile1.txt";

    FileWriter oOut = new FileWriter(sFileName);
    oOut.write("Alt\tAzimuth\tSubquad\tLight\tX\tY\tHeight\n");
    oOut.write("20\t5\t30311\t0\t125\t325\t1.25\n");
    oOut.write("5\t180\t30311\t0\t225\t425\t1.75\n");
    oOut.write("25\t345\t301\t0\t225");

    oOut.close();

    return sFileName;
  }
}
