package gliparameterizer;

import junit.framework.*;

/**
 * <p>Title: GLI Light Ray Tracer</p>
 * <p>Description: Determines which trees intercept given light rays.</p>
 * <p>Copyright: Copyright (c) 2004 Charles D. Canham</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */

public class TestGLICalcs extends TestCase {
  private GLICalcs main = null;

  protected void setUp() throws Exception {
    super.setUp();
    main = new GLICalcs();
  }

  protected void tearDown() throws Exception {
    main = null;
    super.tearDown();
  }

  public void testGetAzimuthAngle() {
    //Zero case
    double fFromX = 0.0;
    double fFromY = 0.0;
    double fToX = 0.0;
    double fToY = 0.0;
    double expectedReturn = 0.0;
    double actualReturn = main.GetAzimuthAngle(fFromX, fFromY, fToX, fToY);
    assertEquals("return value", expectedReturn, actualReturn, 0.0001);

    fFromX = 225;
    fFromY = 321;

    //Between 1.5 and 2 PI
    fToX = 220.92;
    fToY = 426.3;
    expectedReturn = 6.244458241;
    actualReturn = main.GetAzimuthAngle(fFromX, fFromY, fToX, fToY);
    assertEquals("return value", expectedReturn, actualReturn, 0.0001);

    //1.5 PI
    fToX = 100;
    fToY = 321;
    expectedReturn = 4.71238898;
    actualReturn = main.GetAzimuthAngle(fFromX, fFromY, fToX, fToY);
    assertEquals("return value", expectedReturn, actualReturn, 0.0001);

    //Between PI and 1.5PI
    fToX = 113.2;
    fToY = 231.9;
    expectedReturn = 4.039505148;
    actualReturn = main.GetAzimuthAngle(fFromX, fFromY, fToX, fToY);
    assertEquals("return value", expectedReturn, actualReturn, 0.0001);

    //PI
    fToX = 225;
    fToY = 12;
    expectedReturn = 3.141592654;
    actualReturn = main.GetAzimuthAngle(fFromX, fFromY, fToX, fToY);
    assertEquals("return value", expectedReturn, actualReturn, 0.0001);

    //Between 0.5PI and PI
    fToX = 629.1;
    fToY = 0.2;
    expectedReturn = 2.241783919;
    actualReturn = main.GetAzimuthAngle(fFromX, fFromY, fToX, fToY);
    assertEquals("return value", expectedReturn, actualReturn, 0.0001);

    //0.5 PI
    fToX = 230;
    fToY = 321;
    expectedReturn = 1.570796327;
    actualReturn = main.GetAzimuthAngle(fFromX, fFromY, fToX, fToY);
    assertEquals("return value", expectedReturn, actualReturn, 0.0001);

    //Between 0 and 0.5 PI
    fToX = 226;
    fToY = 322;
    expectedReturn = 0.785398163;
    actualReturn = main.GetAzimuthAngle(fFromX, fFromY, fToX, fToY);
    assertEquals("return value", expectedReturn, actualReturn, 0.0001);

    //0
    fToX = 225;
    fToY = 431.6;
    expectedReturn = 0;
    actualReturn = main.GetAzimuthAngle(fFromX, fFromY, fToX, fToY);
    assertEquals("return value", expectedReturn, actualReturn, 0.0001);
  }

  public void testGetDistance() {
    double fFromX = 0.0;
    double fFromY = 0.0;
    double fToX = 0.0;
    double fToY = 0.0;
    double expectedReturn = 0.0;
    double actualReturn = main.GetDistance(fFromX, fFromY, fToX, fToY);
    assertEquals("return value", expectedReturn, actualReturn, 0.0001);

    fFromX = 225;
    fFromY = 321;


    fToX = 220.92;
    fToY = 426.3;
    expectedReturn = 105.3790131;
    actualReturn = main.GetDistance(fFromX, fFromY, fToX, fToY);
    assertEquals("return value", expectedReturn, actualReturn, 0.0001);

    fToX = 100;
    fToY = 321;
    expectedReturn = 125;
    actualReturn = main.GetDistance(fFromX, fFromY, fToX, fToY);
    assertEquals("return value", expectedReturn, actualReturn, 0.0001);

    fToX = 113.2;
    fToY = 231.9;
    expectedReturn = 142.9617082;
    actualReturn = main.GetDistance(fFromX, fFromY, fToX, fToY);
    assertEquals("return value", expectedReturn, actualReturn, 0.0001);

    fToX = 225;
    fToY = 12;
    expectedReturn = 309;
    actualReturn = main.GetDistance(fFromX, fFromY, fToX, fToY);
    assertEquals("return value", expectedReturn, actualReturn, 0.0001);

    fToX = 629.1;
    fToY = 0.2;
    expectedReturn = 515.9548914;
    actualReturn = main.GetDistance(fFromX, fFromY, fToX, fToY);
    assertEquals("return value", expectedReturn, actualReturn, 0.0001);

    fToX = 230;
    fToY = 321;
    expectedReturn = 5;
    actualReturn = main.GetDistance(fFromX, fFromY, fToX, fToY);
    assertEquals("return value", expectedReturn, actualReturn, 0.0001);

    fToX = 226;
    fToY = 322;
    expectedReturn = 1.414213562;
    actualReturn = main.GetDistance(fFromX, fFromY, fToX, fToY);
    assertEquals("return value", expectedReturn, actualReturn, 0.0001);

    fToX = 225;
    fToY = 431.6;
    expectedReturn = 110.6;
    actualReturn = main.GetDistance(fFromX, fFromY, fToX, fToY);
    assertEquals("return value", expectedReturn, actualReturn, 0.0001);
  }

}
