package gliparameterizer;

import junit.framework.*;

/**
 * <p>Title: GLI Light Ray Tracer</p>
 * <p>Description: Determines which trees intercept given light rays.</p>
 * <p>Copyright: Copyright (c) 2004 Charles D. Canham</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */

public class TestGLIRay
    extends TestCase {

  protected void setUp() throws Exception {
    super.setUp();
  }

  protected void tearDown() throws Exception {
    super.tearDown();
  }

  public void testGLIRay() throws Exception {
    GLIRay oRay;
    double fX = 24.1;
    double fY = 265.0;
    double fHeight = 1.6;
    double fAzimuth = 26.5;
    double fAltitude = 60.0;
    String sSubquad = "test";
    String sLight = "test";
    oRay = new GLIRay(fX, fY, fHeight, fAzimuth, fAltitude, sSubquad, sLight);
    assertEquals(24.1, oRay.m_fX, 0.0001);
    assertEquals(265.0, oRay.m_fY, 0.0001);
    assertEquals(1.6, oRay.m_fHeight, 0.0001);
    assertEquals(0.4625122, oRay.m_fAzimuth, 0.0001);
    assertEquals(1.047197551, oRay.m_fAltitude, 0.0001);

    //Error processing
    try {
      fX = -24.1;
      fY = 265.0;
      fHeight = 1.6;
      fAzimuth = 26.5;
      fAltitude = 60.0;
      oRay = new GLIRay(fX, fY, fHeight, fAzimuth, fAltitude, sSubquad, sLight);
      fail("GLIRay constructor didn't catch negative X value.");
    }
    catch (Exception e) {
      ;
    }

    try {
      fX = 24.1;
      fY = -265.0;
      fHeight = 1.6;
      fAzimuth = 26.5;
      fAltitude = 60.0;
      oRay = new GLIRay(fX, fY, fHeight, fAzimuth, fAltitude, sSubquad, sLight);
      fail("GLIRay constructor didn't catch negative Y value.");
    }
    catch (Exception e) {
      ;
    }

    try {
      fX = 24.1;
      fY = 265.0;
      fHeight = -1.6;
      fAzimuth = 26.5;
      fAltitude = 60.0;
      oRay = new GLIRay(fX, fY, fHeight, fAzimuth, fAltitude, sSubquad, sLight);
      fail("GLIRay constructor didn't catch negative height value.");
    }
    catch (Exception e) {
      ;
    }

    try {
      fX = 24.1;
      fY = 265.0;
      fHeight = 1.6;
      fAzimuth = -26.5;
      fAltitude = 60.0;
      oRay = new GLIRay(fX, fY, fHeight, fAzimuth, fAltitude, sSubquad, sLight);
      fail("GLIRay constructor didn't catch negative azimuth value.");
    }
    catch (Exception e) {
      ;
    }

    try {
      fX = 24.1;
      fY = 265.0;
      fHeight = 1.6;
      fAzimuth = 26.5;
      fAltitude = -60.0;
      oRay = new GLIRay(fX, fY, fHeight, fAzimuth, fAltitude, sSubquad, sLight);
      fail("GLIRay constructor didn't catch negative altitude value.");
    }
    catch (Exception e) {
      ;
    }
  }

}
