package gliparameterizer;

import junit.framework.*;

/**
 * <p>Title: GLI Light Ray Tracer</p>
 * <p>Description: Determines which trees intercept given light rays.</p>
 * <p>Copyright: Copyright (c) 2004 Charles D. Canham</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 */

public class TestTree
    extends TestCase {

  protected void setUp() throws Exception {
    super.setUp();
  }

  protected void tearDown() throws Exception {
    super.tearDown();
  }

  /**
   * Tests the Tree constructor, with the allometry calculations.
   */
  public void testConstructor() {
    try {
      //Create a tree population object
      AllTrees oAllTrees = new AllTrees(200, 200);

      //Set it up with allometry data
      oAllTrees.mp_oAllometry.add(new AllometryPackage("ALCLAT", (float) 0.073,
          (float) 1.0, (float) 27.3, (float) 0.025, (float) 0.477));
      oAllTrees.mp_oAllometry.add(new AllometryPackage("BUCCAP", (float) 0.06,
          (float) 0.9, (float) 24.57, (float) 0.048, (float) 0.55));
      oAllTrees.mp_oAllometry.add(new AllometryPackage("CASARB", (float) 0.09,
          (float) 0.8, (float) 30, (float) 0.029, (float) 0.54));
      oAllTrees.mp_oAllometry.add(new AllometryPackage("DACEXC", (float) 0.075,
          (float) 1.1, (float) 23.81, (float) 0.061, (float) 0.576));
      oAllTrees.m_oOther = (new AllometryPackage("OTHER", (float) 0.074,
          (float) 1.5, (float) 23.84, (float) 0.049, (float) 0.497));

      //Create trees
      float fDbh = (float) 16.8;
      float fExpectedHeight = (float) 10.24963503;
      float fExpectedCanrad = (float) 1.2264;
      float fExpectedCanBottom = (float) 5.360559118;
      byte iDamageCode = 0;
      Tree oTree = new Tree("ALCLAT", (float) 14, (float) 35, fDbh, 12, iDamageCode, oAllTrees);
      assertEquals(fExpectedHeight, oTree.m_fHeight, 0.001);
      assertEquals(fExpectedCanrad, oTree.m_fCanopyRadius, 0.001);
      assertEquals(fExpectedCanBottom, oTree.m_fCanopyBottomHeight, 0.001);

      fDbh = (float) 16.9;
      fExpectedHeight = (float) 14.25278379;
      fExpectedCanrad = (float) 0.76427435;
      fExpectedCanBottom = (float) 6.413752707;
      oTree = null;
      oTree = new Tree("BUCCAP", (float) 14, (float) 35, fDbh, 12, iDamageCode, oAllTrees);
      assertEquals(fExpectedHeight, oTree.m_fHeight, 0.001);
      assertEquals(fExpectedCanrad, oTree.m_fCanopyRadius, 0.001);
      assertEquals(fExpectedCanBottom, oTree.m_fCanopyBottomHeight, 0.001);

      fDbh = (float) 21.2;
      fExpectedHeight = (float) 14.50754017;
      fExpectedCanrad = (float) 1.035884174;
      fExpectedCanBottom = (float) 6.673468477;
      oTree = null;
      oTree = new Tree("CASARB", (float) 14, (float) 35, fDbh, 12, iDamageCode, oAllTrees);
      assertEquals(fExpectedHeight, oTree.m_fHeight, 0.001);
      assertEquals(fExpectedCanrad, oTree.m_fCanopyRadius, 0.001);
      assertEquals(fExpectedCanBottom, oTree.m_fCanopyBottomHeight, 0.001);

      fDbh = (float) 17.5;
      fExpectedHeight = (float) 16.08674468;
      fExpectedCanrad = (float) 1.747443397;
      fExpectedCanBottom = (float) 6.820779742;
      oTree = null;
      oTree = new Tree("DACEXC", (float) 14, (float) 35, fDbh, 12, iDamageCode, oAllTrees);
      assertEquals(fExpectedHeight, oTree.m_fHeight, 0.001);
      assertEquals(fExpectedCanrad, oTree.m_fCanopyRadius, 0.001);
      assertEquals(fExpectedCanBottom, oTree.m_fCanopyBottomHeight, 0.001);

      fDbh = (float) 59.3;
      fExpectedHeight = (float) 22.60956131;
      fExpectedCanrad = (float) 33.79198936;
      fExpectedCanBottom = (float) 11.37260934;
      oTree = null;
      oTree = new Tree("ANDINE", (float) 14, (float) 35, fDbh, 12, iDamageCode, oAllTrees);
      assertEquals(fExpectedHeight, oTree.m_fHeight, 0.001);
      assertEquals(fExpectedCanrad, oTree.m_fCanopyRadius, 0.001);
      assertEquals(fExpectedCanBottom, oTree.m_fCanopyBottomHeight, 0.001);

      fDbh = (float) 15;
      fExpectedHeight = (float) 11.05722319;
      fExpectedCanrad = (float) 4.299011514;
      fExpectedCanBottom = (float) 5.561783265;
      oTree = null;
      oTree = new Tree("PREMON", (float) 14, (float) 35, fDbh, 12, iDamageCode, oAllTrees);
      assertEquals(fExpectedHeight, oTree.m_fHeight, 0.001);
      assertEquals(fExpectedCanrad, oTree.m_fCanopyRadius, 0.001);
      assertEquals(fExpectedCanBottom, oTree.m_fCanopyBottomHeight, 0.001);

    }
    catch (java.lang.Exception e) {
      fail(e.getMessage());
    }
  }
}
